using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace WhackAMole
{
	/// <summary>
	/// This class stores information about mouse and speaker calibration.
	/// </summary>
	public class CalibrationInfo
	{
		/// <summary>
		/// The sound device for sound output.
		/// </summary>
		private Device soundDevice;

		/// <summary>
		/// The buffer description for controlling panning and global focus, etc.
		/// </summary>
		private BufferDescription bufferDesc;

		/// <summary>
		/// The mapping between mouse IDs and mole IDs.
		/// </summary>
		/// <remarks>
		/// mouseToMoleMap[i] is the mole ID associated with mouse ID i.
		/// </remarks>
		private int[] mouseToMoleMap;

		/// <summary>
		/// The pan positions for each mole.
		/// </summary>
		private int[] panPositions;

		/// <summary>
		/// The number of moles.
		/// </summary>
		private int numMoles;

		/// <summary>
		/// CalibrationInfo constructor.
		/// </summary>
		/// <param name="numMoles">the number of moles</param>
		/// <param name="map">the mouse-to-mole map</param>
		/// <param name="panPositions">the panning positions for each mole</param>
		/// <param name="dev">the sound output device</param>
		/// <param name="desc">the buffer description for sound output</param>
		public CalibrationInfo(int numMoles, int[] map, int[] panPositions, Device dev, BufferDescription desc)
		{
			this.numMoles = numMoles;
			mouseToMoleMap = map;
			this.panPositions = panPositions;
			soundDevice = dev;
			bufferDesc = desc;
		}

		/// <summary>
		/// SoundDevice property.
		/// </summary>
		/// <value>
		/// Get the sound output device.
		/// </value>
		public Device SoundDevice
		{
			get
			{
				return soundDevice;
			}
		}

		/// <summary>
		/// BufferDesc property.
		/// </summary>
		/// <value>
		/// Get the buffer description.
		/// </value>
		public BufferDescription BufferDesc
		{
			get
			{
				return bufferDesc;
			}
		}

		/// <summary>
		/// MouseToMoleMap property.
		/// </summary>
		/// <value>
		/// Get the mouse-to-mole mapping.  MouseToMoleMap[i] is the mole ID associated with mouse ID i.
		/// </value>
		public int[] MouseToMoleMap
		{
			get
			{
				return mouseToMoleMap;
			}
		}

		/// <summary>
		/// PanPositions property.
		/// </summary>
		/// <value>
		/// Get the pan positions for each mole.
		/// </value>
		public int[] PanPositions
		{
			get
			{
				return panPositions;
			}
		}

		/// <summary>
		/// NumMoles property.
		/// </summary>
		/// <value>
		/// Get the number of moles.
		/// </value>
		public int NumMoles
		{
			get
			{
				return numMoles;
			}
		}
	}
}