using System.Diagnostics;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace WhackAMole
{
	/// <summary>
	/// A class representing a mole.
	/// </summary>
	public class Mole
	{
		/// <summary>
		/// The mole's name.
		/// </summary>
		private string name;

		/// <summary>
		/// Is the mole up?
		/// </summary>
		private bool isUp;

		/// <summary>
		/// The mole sound's panning position.
		/// </summary>
		private int panPosition;

		/// <summary>
		/// The mole's get up sound.
		/// </summary>
		private SecondaryBuffer soundGetUp;

		/// <summary>
		/// The mole's hide sound.
		/// </summary>
		private SecondaryBuffer soundHide;

		/// <summary>
		/// The mole's taunt sound.
		/// </summary>
		private SecondaryBuffer soundTaunt;

		/// <summary>
		/// The mole's hit sound.
		/// </summary>
		private SecondaryBuffer soundHit;

		/// <summary>
		/// If the mole is not currently up, causes the mole to get up.
		/// </summary>
		public void GetUp()
		{
			if(!isUp)
			{
				isUp = true;
				StopAllSounds();
				if(soundGetUp != null)
					soundGetUp.Play(0, BufferPlayFlags.Default);
				Trace.WriteLine(name + " be up!");
			}
		}

		/// <summary>
		/// If the mole is up, causes the mole to start taunting.
		/// </summary>
		public void Taunt()
		{
			if(isUp)
			{
				StopAllSounds();
				if(soundTaunt != null)
					soundTaunt.Play(0, BufferPlayFlags.Looping);
				Trace.WriteLine(name + " be tauntin'!");
			}
		}

		/// <summary>
		/// If the mole is up, causes the mole to hide.
		/// </summary>
		public void Hide()
		{
			if(isUp)
			{
				isUp = false;
				StopAllSounds();
				if(soundHide != null)
					soundHide.Play(0, BufferPlayFlags.Default);
				Trace.WriteLine(name + " be hidin'!");
			}
		}

		/// <summary>
		/// If the mole is up, hits the mole.
		/// </summary>
		public void Hit()
		{
			if(isUp)
			{
				isUp = false;
				StopAllSounds();
				if(soundHit != null)
					soundHit.Play(0, BufferPlayFlags.Default);
				Trace.WriteLine(name + " be hit!");
			}
		}

		/// <summary>
		/// Stops all currently playing mole sounds.
		/// </summary>
		private void StopAllSounds()
		{
			if(soundGetUp != null)
			{
				soundGetUp.Stop();
			}
			if(soundHide != null)
			{
				soundHide.Stop();
			}
			if(soundTaunt != null)
			{
				soundTaunt.Stop();
			}
			if(soundHit != null)
			{
				soundHit.Stop();
			}
		}

		/// <summary>
		/// Sets this mole's speaker panning position.
		/// </summary>
		private void SetPanPositions()
		{
			soundGetUp.Pan = panPosition;
			soundHide.Pan = panPosition;
			soundTaunt.Pan = panPosition;
			soundHit.Pan = panPosition;
		}

		/// <summary>
		/// Name property.
		/// </summary>
		/// <value>
		/// Get or set this mole's name.
		/// </value>
		public string Name
		{
			get
			{
				return name;
			}

			set
			{
				name = value;
			}
		}

		/// <summary>
		/// IsUp property.
		/// </summary>
		/// <value>
		/// Get or set whether or not this mole is up.
		/// </value>
		public bool IsUp
		{
			get
			{
				return isUp;
			}
			set
			{
				// TODO: why is this setter here?  Don't I have a GetUp() method?
				isUp = value;
			}
		}

		/// <summary>
		/// PanPositions property.
		/// </summary>
		/// <value>
		/// Get or set this mole's speaker panning position.
		/// </value>
		public int PanPosition
		{
			get
			{
				return panPosition;
			}
			set
			{
				panPosition = value;
				SetPanPositions();
			}
		}

		/// <summary>
		/// SoundGetUp property.
		/// </summary>
		/// <value>
		/// Set this mole's get up sound.
		/// </value>
		public SecondaryBuffer SoundGetUp
		{
			set
			{
				soundGetUp = value;
			}
		}

		/// <summary>
		/// SoundHide property.
		/// </summary>
		/// <value>
		/// Set this mole's hide sound.
		/// </value>
		public SecondaryBuffer SoundHide
		{
			set
			{
				soundHide = value;
			}
		}

		/// <summary>
		/// SoundTaunt property.
		/// </summary>
		/// <value>
		/// Set this mole's taunt sound.
		/// </value>
		public SecondaryBuffer SoundTaunt
		{
			set
			{
				soundTaunt = value;
			}
		}

		/// <summary>
		/// SoundHit property.
		/// </summary>
		/// <value>
		/// Set this mole's hit sound.
		/// </value>
		public SecondaryBuffer SoundHit
		{
			set
			{
				soundHit = value;
			}
		}
	}
}