using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace WhackAMole
{
	/// <summary>
	/// Modal dialog box for setting up panning positions for each mole.
	/// </summary>
	public class SpeakerCalibrationDialog : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TrackBar panPositionSlider;
		private System.Windows.Forms.TextBox panValueTextBox;
		private System.Windows.Forms.Button OK;
		private System.Windows.Forms.Button Cancel;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// The number of moles that have been calibrated for speaker panning.
		/// </summary>
		private int molesCalibrated = 0;

		/// <summary>
		/// The number of moles to calibrate.
		/// </summary>
		private int numMoles = 0;

		/// <summary>
		/// Sound device for sound output.
		/// </summary>
		private Device device;

		/// <summary>
		/// Buffer description for panning control, global focus, etc.
		/// </summary>
		private BufferDescription desc;

		/// <summary>
		/// The sound being played during panning calibration.
		/// </summary>
		private SecondaryBuffer buffer;

		private System.Windows.Forms.Label instructionLabel;

		/// <summary>
		/// The panning positions for each mole.
		/// </summary>
		private int[] panPositions;

		public SpeakerCalibrationDialog(int numMoles, Device device, BufferDescription desc)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.numMoles = numMoles;
			this.device = device;
			this.desc = desc;

			// create space for pan positions
			panPositions = new int[numMoles];

			instructionLabel.Text = "Please move the slider until the sound seems to be coming from where Mole 0 is." +
				"  Press Cancel to use default values.";

			// play the sound to use for panning calibration
			buffer = new SecondaryBuffer(@"../sounds/laugh.wav", desc, device);
			buffer.Play(0, BufferPlayFlags.Looping);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.instructionLabel = new System.Windows.Forms.Label();
			this.panPositionSlider = new System.Windows.Forms.TrackBar();
			this.panValueTextBox = new System.Windows.Forms.TextBox();
			this.OK = new System.Windows.Forms.Button();
			this.Cancel = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.panPositionSlider)).BeginInit();
			this.SuspendLayout();
			// 
			// instructionLabel
			// 
			this.instructionLabel.Location = new System.Drawing.Point(8, 8);
			this.instructionLabel.Name = "instructionLabel";
			this.instructionLabel.Size = new System.Drawing.Size(504, 24);
			this.instructionLabel.TabIndex = 0;
			this.instructionLabel.Text = "label1";
			// 
			// panPositionSlider
			// 
			this.panPositionSlider.LargeChange = 100;
			this.panPositionSlider.Location = new System.Drawing.Point(8, 56);
			this.panPositionSlider.Maximum = 10000;
			this.panPositionSlider.Minimum = -10000;
			this.panPositionSlider.Name = "panPositionSlider";
			this.panPositionSlider.Size = new System.Drawing.Size(504, 45);
			this.panPositionSlider.SmallChange = 20;
			this.panPositionSlider.TabIndex = 1;
			this.panPositionSlider.TickFrequency = 500;
			this.panPositionSlider.Scroll += new System.EventHandler(this.panPositionSlider_Scroll);
			// 
			// panValueTextBox
			// 
			this.panValueTextBox.Location = new System.Drawing.Point(16, 120);
			this.panValueTextBox.Name = "panValueTextBox";
			this.panValueTextBox.ReadOnly = true;
			this.panValueTextBox.Size = new System.Drawing.Size(72, 20);
			this.panValueTextBox.TabIndex = 2;
			this.panValueTextBox.Text = "0";
			// 
			// OK
			// 
			this.OK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.OK.Location = new System.Drawing.Point(344, 120);
			this.OK.Name = "OK";
			this.OK.Size = new System.Drawing.Size(72, 24);
			this.OK.TabIndex = 3;
			this.OK.Text = "OK";
			this.OK.Click += new System.EventHandler(this.OK_Click);
			// 
			// Cancel
			// 
			this.Cancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.Cancel.Location = new System.Drawing.Point(432, 120);
			this.Cancel.Name = "Cancel";
			this.Cancel.Size = new System.Drawing.Size(72, 24);
			this.Cancel.TabIndex = 4;
			this.Cancel.Text = "Cancel";
			this.Cancel.Click += new System.EventHandler(this.Cancel_Click);
			// 
			// SpeakerCalibrationDialog
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(520, 158);
			this.Controls.Add(this.Cancel);
			this.Controls.Add(this.OK);
			this.Controls.Add(this.panValueTextBox);
			this.Controls.Add(this.panPositionSlider);
			this.Controls.Add(this.instructionLabel);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Name = "SpeakerCalibrationDialog";
			this.Text = "SpeakerCalibrationDialog";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.SpeakerCalibrationDialog_Closing);
			((System.ComponentModel.ISupportInitialize)(this.panPositionSlider)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void panPositionSlider_Scroll(object sender, System.EventArgs e)
		{
			panValueTextBox.Text = panPositionSlider.Value.ToString();
			buffer.Pan = panPositionSlider.Value;
		}

		/// <summary>
		/// Cancel button handler.  Sets up default panning positions.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Cancel_Click(object sender, System.EventArgs e)
		{
			// if the user cancelled speaker calibration, set up some reasonable
			// default values for the pan positions

			// hard left
			panPositions[0] = -10000;

			// hard right
			panPositions[numMoles - 1] = 10000;

			if(numMoles == 3)
			{
				panPositions[1] = 0;
			}
			else
			{
				// create a "gradient" between -1000 and 1000 for the remaining moles
				for(int i = 1; i < numMoles - 1; i++)
				{
					panPositions[i] = -1000 + (i - 1) * (2000 / (numMoles - 3));
				}
			}
			DialogResult = DialogResult.Cancel;
		}

		/// <summary>
		/// OK button handler.  Accepts panning position for current mole being calibrated.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OK_Click(object sender, System.EventArgs e)
		{
			// set up the pan position for the current mole
			panPositions[molesCalibrated] = panPositionSlider.Value;

			molesCalibrated++;

            // display instructions for next mole
			instructionLabel.Text = "Please move the slider until the sound seems to be coming from where Mole " + molesCalibrated + " is." +
				"  Press Cancel to use default values.";

			if(molesCalibrated == numMoles)
			{
				// we're done calibrating
				DialogResult = DialogResult.OK;
			}
		}

		private void SpeakerCalibrationDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if(buffer != null)
			{
				// stop playing the sound when this form is closing
				buffer.Stop();
			}
		}

		/// <summary>
		/// PanPositions property
		/// </summary>
		/// <value>
		/// Get the pan positions for each mole.
		/// </value>
		public int[] PanPositions
		{
			get
			{
				return panPositions;
			}
		}
	}
}
